(*#line 510 "wordgraph.nw"*)signature ppWORDGRAPH =
    sig
        structure W : WORDGRAPH

        type dawg = W.dawg
        type state = W.state
        type token = W.token

        type dotoption = string
        val default : dotoption

        val displayIndex : string -> dotoption -> OS.Process.status
        val displayDawgs : string -> dotoption -> OS.Process.status
        val displayGraphs : dawg list -> dotoption -> OS.Process.status

        val showIndex : string -> dotoption -> string -> OS.Process.status
        val showDawg : string -> dotoption -> string -> OS.Process.status
        val showGraphs : dawg list -> string -> string -> OS.Process.status

        val viewgraph : string -> OS.Process.status
    end

(*#line 556 "wordgraph.nw"*)structure ppWordgraph : ppWORDGRAPH =
    struct
        structure W = Wordgraph

        type dawg = W.dawg
        type token = W.token 
        type state = W.state

        type pda = {start : state list, 
                    final : state list,
                    trans : (state * (token * bool) * state) list}
            
        fun dawgToPartialAutomaton (G:dawg) : pda =
            let
                val states = #states G
                val tokens = #tokens G
                val r = W.reject
                val accepting = List.filter (fn q => not (q = r)) states

                val Trs = ref [] : (int * (token * bool) * int) list ref

                fun addTr (a,p:state) = 
                    let 
                        val q = #trans G (p,a)
                    in 
                        if p = r orelse q = r then () 
                        else Trs := ((p,a,q)::(!Trs))
                    end

                val _ = List.app (fn p => 
                                  (List.app (fn a => addTr ((a,true),p)) (#tokens G)))
                        accepting 
                val _ = List.app (fn p => 
                                  (List.app (fn a => addTr ((a,false),p)) (#tokens G)))
                        accepting 
                val transitions = (!Trs)
            in
                {start = [0], 
                 final = map (fn s:state => s) accepting,
                 trans = transitions}
            end

        fun start ({start = S,...}:pda) = S 
        and final ({final = F,...}:pda) = F
        and trans ({trans = T,...}:pda) = T

        fun node i = "node"^Int.toString i
        fun toString (p::q::rest) = Int.toString p^","^(toString (q::rest))
          | toString (p::nil) = Int.toString p
          | toString nil = " "

        fun drawIndex (M,Positions) os =
            let
                fun write w = TextIO.output (os, w)

                fun defineNode (i,oldstates) = 
                    write (node i^" [label = \""^(toString oldstates)^"\"];\n")

                fun drawTransition (i,(a,_),j) = 
                    write (node i ^ " -> " ^ (node j) ^ 
                           " [label = " ^ (Char.toString a) ^ "];\n")

                fun drawInitialState i = write (node i ^ " [shape = diamond];\n")

            in 
                app defineNode Positions;
                app drawTransition (trans M);
                app drawInitialState (start M)
            end

        fun drawIndex' (M,Positions) os =
            let
                fun write w = TextIO.output (os, w)

                fun drawInitialState i = write (node i ^ " [shape = diamond];\n")

                fun defineNode (i,oldstates) = 
                    write (node i^" [label = \""^(toString oldstates)^"\"];\n")

                fun drawTransition (i,(a,false),j) = 
                    write (node i ^ " -> " ^ (node j) ^ 
                           " [style = dashed, label = " ^ (Char.toString a) ^ "];\n")
                  | drawTransition (i,(a,true),j) = 
                    write (node i ^ " -> " ^ (node j) ^ 
                           " [style = solid, label = " ^ (Char.toString a) ^ "];\n")
            in 
                app defineNode Positions;
                app drawTransition (trans M);
                app drawInitialState (start M)
            end

        fun drawSuffixLinks (G : dawg) os =
            let 
                val r = W.reject

                fun write w = TextIO.output (os, w)

                fun drawLinks (p::ps) =
                    let 
                        val q = #suffix G p
                    in
                        if p = r orelse q = r then drawLinks ps 
                        else (write (node p ^ " -> " ^ (node q) ^
                                    " [color = blue, style=dotted];\n"); 
                              drawLinks ps)
                        end
                  | drawLinks [] = ()
            in
                drawLinks (#states G) 
            end

        fun drawSinkAndNewNode (G : dawg) os = 
            let 
                fun write w = TextIO.output (os, w)
            in 
                write (node (#sink G)^" [color = orange];\n"); 
                if (#max G > #sink G) then write (node (#max G) ^" [color = red];\n") else ()
            end

        fun statesWithPositions (G : dawg) = 
            map (fn q => (q, rev (#positions G q))) 
            (List.filter (fn q => not (q = W.reject)) (#states G))

        type dotoption = string

        val default =
           "center=true;\nsize=\"6.0,10\";\nratio=2.3"  (* \n size=\"7.5,14\"; *) 

(*#line 689 "wordgraph.nw"*)        fun viewgraph filename =  (* better abstract from previewer gv *)
           (OS.Process.system 
            ("dot -Tps " ^ filename ^ ".dot -o " ^ filename ^ ".ps ");
            OS.Process.system 
                (* ("ghostview -magstep -2 " ^ filename ^ ".ps &")) *)
                ("gv --scale=-2 " ^ filename ^ ".ps &")) 

        fun displayIndex text options = 
           showIndex text options "wordindex.tmp"

        and showIndex text options filename =    
           (dotIndex (W.mkDawg text) options filename; viewgraph filename)

        and dotIndex G options filename =
            let
                val os = TextIO.openOut (filename ^ ".dot")
                fun write w = TextIO.output (os, w)
                val (A,Pos) = (dawgToPartialAutomaton G, statesWithPositions G)
            in
               write ("digraph SubwordIndex {" ^ 
                      (if options = "" then default else options) ^ ";\n");
               drawIndex (A,Pos) os; 
               write "}\n";
               TextIO.closeOut os
            end

(*#line 729 "wordgraph.nw"*)        fun displayGraphs Gs options = 
            showGraphs Gs options "wordgraph.tmp"

        and showGraphs Gs options filename =   
            (dotGraphs Gs options filename; viewgraph filename)

        and dotGraphs Gs options filename =
            let
                val os = TextIO.openOut (filename ^ ".dot")
            in
                app (fn G => dotGraph G os options) Gs;
                TextIO.closeOut os
            end          
        and dotGraph G os options = 
            let
                fun write w = TextIO.output (os, w)
                val (A,Pos) = (dawgToPartialAutomaton G, statesWithPositions G)
            in
                write ("digraph Wordgraph"^(Int.toString (#pos G)) ^ "{" ^
                      (if options = "" then default else options) ^ ";\n");
                drawIndex' (A,Pos) os;
                drawSuffixLinks G os;
                drawSinkAndNewNode G os;
                write "}\n\n"
            end          

        and showDawg text options filename =
            showGraphs [W.mkDawg text] options filename

(*#line 764 "wordgraph.nw"*)       local
          fun dawgs (c::cs) (G::Gs) = 
              dawgs cs (W.updateDawg(c,G) :: (G::Gs))
            | dawgs (c::cs) [] = 
              dawgs cs [W.updateDawg(c,W.initG),W.initG]
            | dawgs [] [] = [W.initG]
            | dawgs [] acc = rev acc
       in
          fun displayDawgs text options = 
             displayGraphs (dawgs (explode text) []) options
       end
   end

